local core = LibStub("AceAddon-3.0"):GetAddon("tomQuest2")
local module = core:NewModule("announce", "AceEvent-3.0", "AceHook-3.0", "LibSink-2.0")
local LA = LibStub("AceLocale-3.0"):GetLocale("tomQuest2-Announce")


--------------------------------------
--- Copy global functions to local ---
--------------------------------------
local format = string.format

local defaults = {
	profile = {
		playerObjectiveProgress = true,
		playerObjectiveComplete = true,
		playerQuestComplete = true,
		playerQuestFailed = true,
		objectiveNotificationWhenComplete = false,
	},
}

local prdb
function module:OnInitialize()
	self.db = core.db:RegisterNamespace("announce", defaults, "Default")
	prdb = self.db.profile
end

module.modName = LA["Quests Announcement"]

local options, output
function module:getOptions()
	if not options then
		options = {
			type = "group",
			name = module.modName,
			desc = module.modName,
			get = function(info) 
				return prdb[info[#info]]
			end,
			set = function(info, value)
				prdb[info[#info]] = value
			end,
			args = {
				playerObjectiveProgress = {
					name = LA["Toggle player quests objectives progress annoucement"],
					type = "toggle",
					width = "full",
					order = 12,
				},
				playerObjectiveComplete = {
					name = LA["Toggle player quests objectives complete announcement"],
					type = "toggle",
					width = "full",
					order = 13,
				},
				playerQuestComplete = {
					name = LA["Toggle player quests complete announcement"],
					type = "toggle",
					width = "full",
					order = 14,
				},
				playerQuestFailed = {
					name = LA["Toggle player quests failed announcement"],
					type = "toggle",
					width = "full",
					order = 15,
				},
			},
		}
	end
	return options, output
end

function module:OnEnable()
	core.RegisterCallback(self, "playerQuestObjectiveProgress")
	core.RegisterCallback(self, "playerQuestComplete")
	core.RegisterCallback(self, "playerQuestFailed")
end

function module:OnDisable()
	core.UnregisterAllCallbacks(self)
end

function module:playerQuestObjectiveProgress(event, firstRun, commHandler, uid, objective, got, nGot)
	if nGot == core.questsObjectives[uid][objective].nNeed and prdb.playerObjectiveComplete and not firstRun and GetNumPartyMembers() > 0 then
		if not prdb.objectiveNotificationWhenComplete and core.quests[uid].party[core.playerName].complete ~= 0 then return end
		SendChatMessage(format(LA["(%s) %s : finished"], core.quests[uid].title, core.questsObjectives[uid][objective].desc), "PARTY")
	elseif prdb.playerObjectiveProgress and not firstRun and nGot > 0 and GetNumPartyMembers() > 0 then
		SendChatMessage(format("(%s) %s : %s/%s", core.quests[uid].title, core.questsObjectives[uid][objective].desc , got, core.questsObjectives[uid][objective].need), "PARTY")
	end
end

function module:playerQuestComplete(event, firstRun, commHandler, uid)
	if prdb.playerQuestComplete and not firstRun and GetNumPartyMembers() > 0 then
		SendChatMessage(format(LA["(%s) : finished"], core.quests[uid].title), "PARTY")
	end
end

function module:playerQuestFailed(event, firstRun, commHandler, uid)
	if prdb.playerQuestFailed and not firstRun and GetNumPartyMembers() > 0 then
		SendChatMessage(format(LA["(%s) : failed"], core.quests[uid].title), "PARTY")
	end
end